﻿<#
.SYNOPSIS
	Queries Lync Server 2010/2013 pools and retrieves what Cumulative Update is installed on each server in the pool.

.DESCRIPTION
	Queries Lync Server 2010/2013 pools and retrieves what Cumulative Update is installed on each server in the pool.

.NOTES
  Version      	   		: 1.0 - See changelog at http://www.ehloworld.com/2642
  Wish list						: Better error trapping
  Rights Required			: CSAdministrator and local admin rights on the target computers
  Sched Task Required	: No
  Lync Server Version	: 2010 and 2013
  Author(s)    				: Dave Howe
  										: Pat Richard
	Email/Blog/Twitter	: daveh@microsoft.com
												pat@innervation.com 	http://www.ehloworld.com @patrichard
	Dedicated Post			: http://www.ehloworld.com/2641
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is provided AS IS
												without warranty of any kind. I disclaim all implied warranties including, without limitation, any implied
												warranties of merchantability or of fitness for a particular purpose. The entire risk arising out of the use
												or performance of the sample scripts and documentation remains with you. In no event shall I be liable for
												any damages whatsoever (including, without limitation, damages for loss of business profits, business
												interruption, loss of business information, or other pecuniary loss) arising out of the use of or inability
												to use the script or documentation.
	Acknowledgements 		:
	Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					: PSRemoting must be enabled on the servers that are queried. It's enabled by default on Windows Server 2012
  											and later. For information on enabling PSRemoting, see http://technet.microsoft.com/en-us/library/hh849694.aspx
	Known issues				: Edge servers that aren't domain joined will show as "PSRemoting Failure".

.EXAMPLE
	.\Get-CsUpdateVersion.ps1

	Description
	-----------
	Runs the script and queries all pools in the environment.

.EXAMPLE
	.\Get-CsUpdateVersion.ps1 -PoolFqdn pool.contoso.com

	Description
	-----------
	Runs the script and queries just the pool specified.
#>
#Requires -Version 2.0

[CmdletBinding(SupportsShouldProcess = $True)]
param (
	# Specifies a single pool to query
	[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)]
	[string] $PoolFqdn
)

function Set-ModuleStatus {
	[CmdletBinding(SupportsShouldProcess = $True)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")]
		[ValidateNotNullOrEmpty()]
		[string] $name
	)
	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		if (!(Get-Module -name "$name")) {
			if (Get-Module -ListAvailable | Where-Object Name -eq "$name") {
				Import-Module -Name "$name"
				# module was imported
				# return $true
			} else {
				# module was not available
				# return $false
			}
		} else {
			# Write-Output "$_ module already imported"
			# return $true
		}
	} # end PROCESS
} # end function Set-ModuleStatus

Set-ModuleStatus Lync
if ($PoolFqdn){
	$services = @(Get-CsService | Where-Object {$_.version -ge 5 -and $_.PoolFqdn -eq $PoolFqdn})
}else{
	$services = @(Get-CsService | Where-Object {$_.version -ge 5})
}
$AllowedServices = @("ApplicationServer","ArchivingServer","BackupServer","CentralManagement","ConferencingServer","Director","EdgeServer","MediationServer","MonitoringServer","PersistentChatServer","Registrar","UserServer","WebServer")

[string] $ComputerFqdn = [System.Net.Dns]::GetHostByName(($env:ComputerName)).HostName

foreach ($service in $services)	{
	if ($AllowedServices -icontains $service.role) {
		$pools += @($service.PoolFqdn)
	}
}

$pools = $pools | Select-Object -Unique | Sort-Object
Write-Output "`n`n"
foreach ($Fqdn in $pools)	{
	$computers = @((Get-CsPool $Fqdn).computers | Sort-Object)

	if ($computers) {
		Write-Host "Pool: $Fqdn `($($computers.count) servers`)`n" -ForegroundColor white

		foreach ($computer in $computers) {
			if(!(Test-Connection -ComputerName $computer -BufferSize 16 -Count 1 -ErrorAction SilentlyContinue -Quiet)) {
				Write-Host "      $computer (" -ForegroundColor white -NoNewline
				Write-Host "offline" -ForegroundColor red -NoNewline
				Write-Host ")" -ForegroundColor white
			} else {
				if (!($computer -eq $ComputerFqdn)){
					$version = Invoke-Command -ScriptBlock { @((Get-ChildItem HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall | ForEach-Object { Get-ItemProperty $_.pspath } | Where-Object { $_.DisplayName -imatch "Microsoft Lync Server" }).DisplayVersion | Sort-Object -Descending | Select-Object -First 1)} -ComputerName $computer -ErrorAction SilentlyContinue -WarningAction SilentlyContinue
				}else{
					$version = (Get-ChildItem HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall | ForEach-Object { Get-ItemProperty $_.pspath } | Where-Object { $_.DisplayName -imatch "Microsoft Lync Server 2013" }).DisplayVersion | Sort-Object -Descending | Select-Object -First 1
				}
				Write-Host "      $computer (" -ForegroundColor white -NoNewline
				if (!$version) {
					Write-Host "PSRemoting failure" -ForegroundColor red -NoNewline
					Write-Host ")" -ForegroundColor white
				} else {
					switch ([string] $version) {
						# Lync Server 2010 Cumulative Updates
						"4.0.7577.0" {$FriendlyResult = "Lync Server 2010 RTM - October 2010"}
						"4.0.7577.108" {$FriendlyResult = "Lync Server 2010 CU1 - January 2011"}
						"4.0.7577.118" {$FriendlyResult = "Lync Server 2010 post-CU1 - February 2011"}
						"4.0.7577.137" {$FriendlyResult = "Lync Server 2010 CU2 - April 2011"}
						"4.0.7577.166" {$FriendlyResult = "Lync Server 2010 CU3 - July 2011"}
						"4.0.7577.183" {$FriendlyResult = "Lync Server 2010 CU4 - November 2011"}
						"4.0.7577.184" {$FriendlyResult = "Lync Server 2010 post-CU4 - December 2011"}
						"4.0.7577.190" {$FriendlyResult = "Lync Server 2010 CU5 - February 2012"}
						"4.0.7577.197" {$FriendlyResult = "Lync Server 2010 post-CU5 - March 2012"}
						"4.0.7577.199" {$FriendlyResult = "Lync Server 2010 CU6 - June 2012"}
						"4.0.7577.203" {$FriendlyResult = "Lync Server 2010 CU7 - October 2012"}
						"4.0.7577.205" {$FriendlyResult = "Lync Server 2010 post-CU7 - October 2012"}
						"4.0.7577.216" {$FriendlyResult = "Lync Server 2010 CU8 - March 2013"}
						"4.0.7577.217" {$FriendlyResult = "Lync Server 2010 CU9 - July 2013"}
						"4.0.7577.223" {$FriendlyResult = "Lync Server 2010 CU10 - October 2013"}
						"4.0.7577.225" {$FriendlyResult = "Lync Server 2010 CU11 - January 2014"}

						# Lync Server 2013 Cumulative Updates
						"5.0.8308.0" {$FriendlyResult = "Lync Server 2013 RTM"}
						"5.0.8308.291" {$FriendlyResult = "Lync Server 2013 CU1 - February 2013"}
						# 5.0.8308.3 (Debugging tools)
						"5.0.8308.420" {$FriendlyResult = "Lync Server 2013 CU2 - July 2013"}
						"5.0.8308.556" {$FriendlyResult = "Lync Server 2013 CU3 - October 2013"}
						# 5.0.8308.553 (LRS portal)
						"5.0.8308.577" {$FriendlyResult = "Lync Server 2013 CU4 - January 2014"}
						default {}
					}
					Write-Host $version -ForegroundColor yellow -NoNewline
					if ($FriendlyResult){
						Write-Host " [$FriendlyResult]" -ForegroundColor white -NoNewline
					}
					Write-Host ")" -ForegroundColor white
				}
			}
		}
	}else{
		Write-Output "No computers found"
	}
	Write-Host
}

# SIG # Begin signature block
# MIIQBQYJKoZIhvcNAQcCoIIP9jCCD/ICAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUPwcYSAifTXwYpGjvGZqqwcvm
# VvSggg1KMIIGnzCCBYegAwIBAgIQCSu3nvOdPAJrDHL+XciEEzANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMB4XDTEzMDgyODAwMDAwMFoXDTE0MDkwNDEy
# MDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBTdGVy
# bGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYDVQQD
# ExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEA8dXC+TSUZsMisaialKJLat7AnmOF0HE+U49duaBJp/N4IXhjUZM9sXRkR/X0
# KAbNauNpNrs3TLKNpO7/RUinzrzsg8IhtBVXSrK7c0HDqJ7vG181it2qQO1yB9uD
# UV8frjprnNcR1w0qW9wFqclKlkyOxgiCANVoT6qXO9IxPZT28h+tMfBCsU7sZALs
# 3kPRMCQ1ZywngVSqEQ9rBYV3jW4EgvPEOmuhtnwgVB1Alr2V1dVAtbNxw1A7fQ8w
# J6EQSNAPVJQq7304QKKgGaTXtA2TwAjO5+m9ZltD/s0wXWYs0V1r8pOJOo1r17l4
# z91OWLPP+yfxvlnm5F1IXZxCbQIDAQABo4IDOTCCAzUwHwYDVR0jBBgwFoAUe2jO
# KarAF75JeuHlP9an90WPNTIwHQYDVR0OBBYEFB9N9j4QNDPcSscWMAVDKCOcXS/G
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBzBgNVHR8EbDBq
# MDOgMaAvhi1odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy0yMDEx
# YS5jcmwwM6AxoC+GLWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNz
# LTIwMTFhLmNybDCCAcQGA1UdIASCAbswggG3MIIBswYJYIZIAYb9bAMBMIIBpDA6
# BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBv
# c2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAg
# AG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBz
# AHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABo
# AGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABo
# AGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBu
# AHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAg
# AGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQBy
# AGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjCBggYIKwYBBQUHAQEE
# djB0MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wTAYIKwYB
# BQUHMAKGQGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENvZGVTaWduaW5nQ0EtMS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0B
# AQUFAAOCAQEABTcbxhPxSJP7AKrF23wXbgelEGzjpewmm0BnHO9/V+9K4UE/3i5Z
# e3bX7OvUb/55zCHT4pvZl8/i8tuvFnrWZZcjGljT3W9SIVOuKNdlItXC8dvP67Hz
# OMZvSbdI5MU6sjd3vyYKznTMHG1Mz/x+BMh6rEnGsIaY4r8r4fkNfJTyTkww8AWh
# I16Pno3dWgKhfploPODtMxczJKA15VDFhQqM3oHHMojasXo5uGb5jlGiDFkZ3mDa
# /+gLUsJUjAzF27ZIy/jRgp0eMkqlgdxGa+gZUK73Ue6aTP2LW9SL/1fLYl8QkwRs
# MWG1faGdureT3Oh0EHH6UhREB/SxLJ+xLTCCBqMwggWLoAMCAQICEA+oSQYV1wCg
# viF2/cXsbb0wDQYJKoZIhvcNAQEFBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UE
# AxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTExMDIxMTEyMDAwMFoX
# DTI2MDIxMDEyMDAwMFowbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMTCCASIwDQYJKoZIhvcNAQEB
# BQADggEPADCCAQoCggEBAJx8+aCPCsqJS1OaPOwZIn8My/dIRNA/Im6aT/rO38bT
# JJH/qFKT53L48UaGlMWrF/R4f8t6vpAmHHxTL+WD57tqBSjMoBcRSxgg87e98tzL
# uIZARR9P+TmY0zvrb2mkXAEusWbpprjcBt6ujWL+RCeCqQPD/uYmC5NJceU4bU7+
# gFxnd7XVb2ZklGu7iElo2NH0fiHB5sUeyeCWuAmV+UuerswxvWpaQqfEBUd9YCvZ
# oV29+1aT7xv8cvnfPjL93SosMkbaXmO80LjLTBA1/FBfrENEfP6ERFC0jCo9dAz0
# eotyS+BWtRO2Y+k/Tkkj5wYW8CWrAfgoQebH1GQ7XasCAwEAAaOCA0MwggM/MA4G
# A1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzCCAcMGA1UdIASCAbow
# ggG2MIIBsgYIYIZIAYb9bAMwggGkMDoGCCsGAQUFBwIBFi5odHRwOi8vd3d3LmRp
# Z2ljZXJ0LmNvbS9zc2wtY3BzLXJlcG9zaXRvcnkuaHRtMIIBZAYIKwYBBQUHAgIw
# ggFWHoIBUgBBAG4AeQAgAHUAcwBlACAAbwBmACAAdABoAGkAcwAgAEMAZQByAHQA
# aQBmAGkAYwBhAHQAZQAgAGMAbwBuAHMAdABpAHQAdQB0AGUAcwAgAGEAYwBjAGUA
# cAB0AGEAbgBjAGUAIABvAGYAIAB0AGgAZQAgAEQAaQBnAGkAQwBlAHIAdAAgAEMA
# UAAvAEMAUABTACAAYQBuAGQAIAB0AGgAZQAgAFIAZQBsAHkAaQBuAGcAIABQAGEA
# cgB0AHkAIABBAGcAcgBlAGUAbQBlAG4AdAAgAHcAaABpAGMAaAAgAGwAaQBtAGkA
# dAAgAGwAaQBhAGIAaQBsAGkAdAB5ACAAYQBuAGQAIABhAHIAZQAgAGkAbgBjAG8A
# cgBwAG8AcgBhAHQAZQBkACAAaABlAHIAZQBpAG4AIABiAHkAIAByAGUAZgBlAHIA
# ZQBuAGMAZQAuMBIGA1UdEwEB/wQIMAYBAf8CAQAweQYIKwYBBQUHAQEEbTBrMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKG
# N2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJv
# b3RDQS5jcnQwgYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwOqA4oDaGNGh0dHA6Ly9j
# cmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwHQYD
# VR0OBBYEFHtozimqwBe+SXrh5T/Wp/dFjzUyMB8GA1UdIwQYMBaAFEXroq/0ksuC
# MS1Ri6enIZ3zbcgPMA0GCSqGSIb3DQEBBQUAA4IBAQB7ch1k/4jIOsG36eepxIe7
# 25SS15BZM/orh96oW4AlPxOPm4MbfEPE5ozfOT7DFeyw2jshJXskwXJduEeRgRNG
# +pw/alE43rQly/Cr38UoAVR5EEYk0TgPJqFhkE26vSjmP/HEqpv22jVTT8nyPdNs
# 3CPtqqBNZwnzOoA9PPs2TJDndqTd8jq/VjUvokxl6ODU2tHHyJFqLSNPNzsZlBjU
# 1ZwQPNWxHBn/j8hrm574rpyZlnjRzZxRFVtCJnJajQpKI5JA6IbeIsKTOtSbaKbf
# KX8GuTwOvZ/EhpyCR0JxMoYJmXIJeUudcWn1Qf9/OXdk8YSNvosesn1oo6WQsQz/
# MYICJTCCAiECAQEwgYMwbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMQIQCSu3nvOdPAJrDHL+XciE
# EzAJBgUrDgMCGgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG
# 9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIB
# FTAjBgkqhkiG9w0BCQQxFgQUYVdJsrDyj6nurxegL0uWpjuZ+CwwDQYJKoZIhvcN
# AQEBBQAEggEAKh21nGSfRelJjmfqlMG84bvyCawGcG9BZfUyRmBafz4DiFUk2bzm
# p4yiP8pHIS5Q4ZtAqXa6aAQItHwSq2X+BTK/7TN9r7dy88JUvD4/Aj8ctSGG5h6l
# /dV1KKhrhJrZKEQe9kSggjp6na8LjTKqhAv93Hv+Z3pZNIq3iQDywhC5X8Ep/jRP
# EZ6645QKIo7uEtyElT+odqXwe6B+WTkXpQn4CKEplG+mToij7KLH1TQyyH6o51Jy
# lZgC/kRBDpRbY8j08NbBhBgt7MbQhg8NdoYYgpSHpltHkWDANSna7ZuglnMLWnnj
# TaXHRSe1aymumkEz/vXyGnPe9vIwiWwM0w==
# SIG # End signature block
